#include "module.h"

#define AUTHOR "Alek"
#define VERSION "0.1B - FATAL"
#define DB_NAME "os_watch.db"
#define DB_UPDATE "5m"

int mLoadData(void);
int mSaveData(int argc, char **argv);

#define hash(nick)      ((tolower((nick)[0])&31)<<5 | (tolower((nick)[1])&31))

/* Watch structure */

typedef struct watchinfo_ Watch;
typedef struct watchdata_ WData;

struct watchinfo_ {
    Watch *prev, *next;
    char *nick;
    int32 time;
    
};

struct watchdata_ {
    WData *prev, *next;
    char *string;
};


Watch *wilists[1024];
WData *wdatalists[1024];

WData *new_data(const char *string){
      WData *wd;
      
      wd = scalloc(1, sizeof(WData));
      wd->string = sstrdup(string);
      
      wd->prev = NULL;
      wd->next = wdatalists[hash(string)];
      
      if (wd->next)
         wd->next->prev = wd;
         wdatalists[hash(string)] = wd;
      return wd;
}

Watch *findwatch(const char *nick){
    Watch *wi;
    
    for (wi = wilists[hash(nick)]; wi; wi = wi->next){
        if (!stricmp(wi->nick, nick))
           return wi;
    }
    
    return NULL;
}

Watch *new_watch(const char *nick){
    Watch *wi;
    
    wi = scalloc(1, sizeof(Watch));
    wi->nick = sstrdup(nick);
    
    wi->prev = NULL;
        wi->next = wilists[hash(nick)];
    if (wi->next)
        wi->next->prev = wi;
        wilists[hash(nick)] = wi;
        return wi;
}

int del_watch(const char *nick){
    Watch *wi;
    if (!(wi = findwatch(nick)))
        return -1;
    
    if (wi->prev)
        wi->prev->next = wi->next;
    else
        wilists[hash(nick)] = wi->next;
    if (wi->next)
        wi->next->prev = wi->prev;
    
    free(wi->nick);
    free(wi);
    return 0;
}

int my_privmsg(char *source, int ac, char **msg);
int my_nick(char *source, int ac, char **msg);
int my_quit(char *source, int ac, char **msg);
int my_watch(User *u);

void AnopeInit(void){
    Message *msg;
    Command *c;
    c = createCommand("WATCH", my_watch, is_services_root, -1, -1, -1, -1, -1);
    moduleAddCommand(OPERSERV, c, MOD_HEAD); 
    msg = createMessage("PRIVMSG", my_privmsg);
    moduleAddMessage(msg, MOD_TAIL);
    msg = createMessage("NICK", my_nick);
    moduleAddMessage(msg, MOD_TAIL);
    msg = createMessage("QUIT", my_quit);
    moduleAddMessage(msg, MOD_TAIL);
    moduleAddAuthor(AUTHOR);
    moduleAddVersion(VERSION);
    mLoadData();
    mSaveData(0, NULL);	
    alog("os_watch: Loaded");
}

void AnopeFini(void){
    mSaveData(0, NULL);
    alog("os_watch: UnLoaded");
}

int my_privmsg(char *source, int ac, char **msg){
    Watch *watch; 
    int i;
    WData *wd;
   if ((watch = findwatch(source))){
        if (watch->time + 90 > time(NULL)){
           for(i = 0; i<1024;i++) {
		      	for(wd = wdatalists[i]; wd; wd = wd->next ) {	
    				if(match_wild_nocase(wd->string,msg[1])){
                        kill_user(s_OperServ, watch->nick, "Spamming");
                        alog(s_OperServ, "\2Possible spammer: %s", watch->nick);
                        del_watch(watch->nick);
         	        }                                   
			    }
       }
    }
    }
    return MOD_CONT;
}

int my_nick(char *source, int ac, char **msg){
    Watch *watch;
    if (!(watch = findwatch(msg[0])))
        watch = new_watch(msg[0]);
        
        watch->time = time(NULL);
    
    return MOD_CONT;
}


int my_watch(User *u){
    char *cmd = strtok(NULL, " ");
    char *string = strtok(NULL, "");
    
    if (!cmd){
        notice(s_OperServ, u->nick, "WATCH ADD <Watch String> to add a new watchs tring");
        notice(s_OperServ, u->nick, "Wildcards (*) accepted");
        return -1;
    }    
    if (!stricmp("ADD", cmd)){
        if (!string) {
            return MOD_CONT;
        } else {
            notice(s_OperServ, u->nick, "Watch string added"); 
            new_data(string);   
        }
    } else if (!stricmp("DEL", cmd)){
        
    }    
    return MOD_CONT;
}


int my_quit(char *source, int ac, char **msg){
    del_watch(source);
    
    return MOD_CONT;   
}

int mSaveData(int argc, char **argv) {
    WData *wd = NULL;
	int i = 0;
	FILE *out;
	if ((out = fopen(DB_NAME, "w")) == NULL) {
		alog("os_watch.so: ERROR! I cant open my data file.");
	} else {		
		for(i = 0; i<1024;i++) {
			for(wd = wdatalists[i]; wd; wd = wd->next ) {
                               if (wd->string)	
    				fprintf(out,"%s\n",wd->string);
			}
		}
		
	fclose(out);
	}
	moduleAddCallback("save me",time(NULL)+dotime(DB_UPDATE),mSaveData,0,NULL);
	return 0;
}

int mLoadData(void) {
	FILE *in;
	char buffer[2000];
	char *string = NULL;
	
	if ((in = fopen(DB_NAME, "r")) == NULL) {
		alog("os_watch.so: Can't read data file (may not exist yet))");
	} else {	
		while (!feof(in)) {
		   fgets(buffer,1500,in);
            	   string = strtok(buffer, "\n");
		   if(string) 
                     new_data(string);
      		}
        fclose(in);
	}
	return 0;
}
